/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _COLLECTION_MODEL_H
#define _COLLECTION_MODEL_H

#include <QVariant>
#include <QString>
#include <QStringList>
#include <QObject>

#include "MediaItem/mediaitem.h"

/*
********************************************************************************
*                                                                              *
*    Class CollectionModel                                                     *
*                                                                              *
********************************************************************************
*/
class CollectionModel : public QObject
{
static CollectionModel* INSTANCE;

Q_OBJECT
  public:
     CollectionModel(QObject *parent = 0);
     ~CollectionModel();
     static CollectionModel* instance() { return INSTANCE; }    
    
     //! custom method
     MediaItemPtr rootItem();
     MediaItemPtr addChild(const MediaItemPtr parent, T_TYPE t);
     MediaItemPtr getItem(const MediaItemPtr,const int childIdx); 
     void clear();
     
     //! get Track method
     QList<MediaItemPtr> getItemChildrenTracks(const MediaItemPtr parent);
     QList<MediaItemPtr> getAlbumChildrenTracksGenre(const MediaItemPtr album,const QString& genre);

     //! get Rating method
     float getItemAutoRating(const MediaItemPtr media);

     //! filtering method
     bool isTrackFiltered(const MediaItemPtr trackItem);
     bool isAlbumFiltered(const MediaItemPtr albumItem);
     bool isArtistFiltered(const MediaItemPtr  artistItem);
     void setFilter(const QString & f) {m_filter_pattern = f;}
    
     
     //! playing status method
     bool itemIsPlaying(const MediaItemPtr item);
     void setItemIsPlaying(const MediaItemPtr item, bool playing);
     
     MediaItemPtr getPlayingTrackItem();
     QString getPlayingTrackItemGenre();
     void updatePlayingItem(const QString& filename);
     void stopPlayingItem();

     //! Favorites status method
     bool itemIsFavorite(const MediaItemPtr item);
     void setItemIsFavorite(const MediaItemPtr item, bool isfavorite);
     
     //! Broken status method
     void updateBrokenItem(const QString filename);

     //! list of MediaItem
     QHash<int, MediaItemPtr> trackItemHash;
     QList<MediaItemPtr>      albumItemList;
     
  signals:
    void signalPlayingStatusChanged();
    void signalFavoriteStatusChanged();    
    void signalPlaycountChanged();
    void modelCleared();

  private:
     MediaItemPtr    m_rootItem;
     int             m_playing_sql_idx;
     QString         m_filter_pattern;     
};

#endif // _COLLECTION_MODEL_H
