/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionModel/collectionpopulator.h"
#include "CollectionModel/collectionmodel.h"
#include "MediaItem/mediaitem.h"
#include "EnginePlayer/engineplayer.h"

#include "Database/database.h"
#include "yarocksettings.h"
#include "utilities.h"
#include "debug.h"

#include <QFileInfo>
#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>
#include <QVariant>

#include <QDateTime>

/*
********************************************************************************
*                                                                              *
*    Class CollectionPopulator                                                 *
*                                                                              *
********************************************************************************
*/
CollectionPopulator::CollectionPopulator()
{
    _collectionModel = CollectionModel::instance();
    _exit            = false;
    
    _populatorTimer = new QTimer();
    connect(_populatorTimer, SIGNAL(timeout()), SIGNAL(populatingRunning()));
}

void CollectionPopulator::run()
{
    Debug::debug() << " --- CollectionPopulator--> Starting model population : " << QTime::currentTime().toString();
    
    /*-----------------------------------------------------------*/
    /* Get connection                                            */
    /* ----------------------------------------------------------*/
    Database db;
    if (!db.connect("CollectionPopulator")) {
        emit populatingFinished();      
        return;
    }

    /*-----------------------------------------------------------*/
    /* Get file count from database                              */
    /* ----------------------------------------------------------*/
    QSqlQuery queryCount("SELECT COUNT(*) FROM `view_tracks`",*db.sqlDb());
    queryCount.next();
    if (queryCount.value(0).toInt() == 0) {
      emit populatingFinished();
      return;
    }
  
    //int _progressMax = queryCount.value(0).toInt();
    int _progress    = 0;
      
      
    /*-----------------------------------------------------------*/
    /* Parse Database and populate model                         */
    /* ----------------------------------------------------------*/       
    _collectionModel->clear();
    _populatorTimer->start(5000);

    MediaItemPtr artistItem = MediaItemPtr(0);
    MediaItemPtr albumItem  = MediaItemPtr(0);
    MediaItemPtr trackItem  = MediaItemPtr(0);

    
    QVariant artist_id = -1;
    QVariant album_id  = -1;
    QVariant track_id  = -1;
    
    QSqlQuery query_1("SELECT artist_id,artist_name,artist_favorite,artist_playcount,artist_rating, \
                              album_id,album_name,album_year,album_cover,album_favorite,album_playcount,album_rating, \
                              id,trackname,filename,number,genre_name,length,albumgain,albumpeakgain,trackgain,trackpeakgain,last_played,playcount,rating \
                       FROM view_tracks \
                       ORDER BY artist_name COLLATE NOCASE ASC,album_year ASC,album_name ASC,album_disc ASC, number ASC",*db.sqlDb());
    while (query_1.next() && !_exit) 
    {
      if(  artist_id != query_1.value(0) ) {
        artist_id = query_1.value(0);

        artistItem = _collectionModel->addChild(_collectionModel->rootItem(), TYPE_ARTIST);
        artistItem->data[ARTIST::id]            =  query_1.value(0);
        artistItem->data[ARTIST::name]          =  query_1.value(1);
        artistItem->data[ARTIST::coverpathlist] =  QStringList();
        artistItem->data[ARTIST::isFavorite]    =  query_1.value(2);
        artistItem->data[ARTIST::playcount]     =  query_1.value(3);
        artistItem->data[ARTIST::rating]        =  query_1.value(4);
        artistItem->data[ARTIST::isUserRating]  =  (query_1.value(4).toFloat() != -1.0) ? true : false;
        artistItem->setParent(_collectionModel->rootItem());
      }
      
      if(  album_id != query_1.value(5) ) {
        album_id = query_1.value(5);

        albumItem = _collectionModel->addChild(artistItem, TYPE_ALBUM);
        albumItem->data[ALBUM::id]            =  query_1.value(5);
        albumItem->data[ALBUM::name]          =  query_1.value(6);
        albumItem->data[ALBUM::year]          =  query_1.value(7);
        albumItem->data[ALBUM::coverpath]     =  query_1.value(8);
        albumItem->data[ALBUM::isFavorite]    =  query_1.value(9);
        albumItem->data[ALBUM::playcount]     =  query_1.value(10);
        albumItem->data[ALBUM::rating]        =  query_1.value(11);
        albumItem->data[ALBUM::isUserRating]  =  (query_1.value(11).toFloat() != -1.0) ? true : false;
        albumItem->setParent(artistItem);
              
        _collectionModel->albumItemList.append(albumItem);
      }    
      
      if(  track_id != query_1.value(12) ) {
        track_id = query_1.value(12);
        
        trackItem = _collectionModel->addChild(albumItem, TYPE_TRACK);
        trackItem->data[TRACK::id]         =  query_1.value(12);
        trackItem->data[TRACK::title]      =  query_1.value(13);
        trackItem->data[TRACK::url]        =  query_1.value(14);
        trackItem->data[TRACK::num]        =  query_1.value(15);
        trackItem->data[TRACK::artist]     =  artistItem->data[ARTIST::name];
        trackItem->data[TRACK::album]      =  albumItem->data[ALBUM::name];
        trackItem->data[TRACK::genre]      =  query_1.value(16);
        trackItem->data[TRACK::duration]   =  query_1.value(17);
        trackItem->data[TRACK::albumGain]  =  query_1.value(18);
        trackItem->data[TRACK::albumPeak]  =  query_1.value(19);
        trackItem->data[TRACK::trackGain]  =  query_1.value(20);
        trackItem->data[TRACK::trackPeak]  =  query_1.value(21);
        trackItem->data[TRACK::lastPlayed] =  !query_1.value(22).isNull() ? query_1.value(22) : -1;
        trackItem->data[TRACK::playcount]  =  query_1.value(23);
        trackItem->data[TRACK::rating]     =  query_1.value(24);
        trackItem->setParent(albumItem);
        _collectionModel->trackItemHash[track_id.toInt()] = trackItem;        

        _progress++;
      }
      
      //emit populatingProgress(  (int) ( (_progress*100)/_progressMax ) );
      //Debug::debug() << " --- CollectionPopulator -->populatingProgress :" <<  (int) ( (_progress*100)/_progressMax ) << " %";
      
    } // end while
    
    /*-----------------------------------------------------------*/
    /* Calculate auto rating                                     */
    /* ----------------------------------------------------------*/    
    for ( int i = 0; i < _collectionModel->rootItem()->childCount(); i++ ) 
    {
      float artist_rating = 0.0;
      MediaItemPtr artist_media = _collectionModel->rootItem()->child(i);

      for (int j = 0; j < artist_media->childCount(); j++) 
      {
         MediaItemPtr album_media = artist_media->child(j);

         float album_rating = _collectionModel->getItemAutoRating(album_media);
         artist_rating += album_rating;
         if(album_media->data.at(ALBUM::isUserRating).toBool() == false)
           album_media->data[ALBUM::rating] = album_rating;
      }
      artist_rating = artist_rating/artist_media->childCount();
      if(artist_media->data.at(ARTIST::isUserRating).toBool() == false)
        artist_media->data[ARTIST::rating] = artist_rating;
    }
    
    _populatorTimer->stop();

    /*-----------------------------------------------------------*/
    /* Update now playing status into model                      */
    /* ----------------------------------------------------------*/      
    QString currentTrack = EnginePlayer::instance()->enginePlayingUrl();
    if(!currentTrack.isEmpty()) {
        _collectionModel->updatePlayingItem(QFileInfo(currentTrack).absoluteFilePath());
    }
    
    /*-----------------------------------------------------------*/
    /* End                                                       */
    /* ----------------------------------------------------------*/ 
    Debug::debug() << " --- CollectionPopulator -->Ending model population :" << QTime::currentTime().toString();

    if(!_exit)
      emit populatingFinished();
}
