/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionModel/plsfilemodel.h"
#include "MediaItem/mediaitem.h"
//#include "debug.h"

#include <QRegExp>

PlsFileModel* PlsFileModel::INSTANCE = 0;

/*
********************************************************************************
*                                                                              *
*    Class PlsFileModel                                                        *
*                                                                              *
********************************************************************************
*/
PlsFileModel::PlsFileModel(QObject *parent) : QObject(parent)
{
    INSTANCE = this;
         
    m_rootItem          = MediaItemPtr(new MediaItem(TYPE_UNKWON));    
    m_filter_pattern    = "";
    m_playing_item_idx  = -1;
}

PlsFileModel::~PlsFileModel()
{
    m_rootItem.reset();
}


MediaItemPtr PlsFileModel::getItem(const MediaItemPtr parent,const int childIdx) 
{
    if(!parent) return MediaItemPtr(0);
    
    if(childIdx < 0 || childIdx >= parent->childCount()) return MediaItemPtr();
    
    return parent->child(childIdx);
}


MediaItemPtr PlsFileModel::rootItem()
{
  return m_rootItem;
}


MediaItemPtr PlsFileModel::addChild(const MediaItemPtr parent, T_TYPE t)
{
  if(!parent) return MediaItemPtr(0);
  
  return parent->addChildren(t);
}


void PlsFileModel::clear()
{
    m_rootItem.reset();
    m_rootItem        = MediaItemPtr(new MediaItem(TYPE_UNKWON));
    
    m_playing_item_idx  = -1;
    
    emit modelCleared();
}

//! ----------------------- Get urls method ------------------------------------
QList<MediaItemPtr> PlsFileModel::getItemChildrenUrls(const MediaItemPtr parent)
{
    QList<MediaItemPtr> result;
    
    if(!parent) return result;

    MediaItemPtr item = parent;
    
    if (item->type == TYPE_TRACK || item->type == TYPE_STREAM) {
      if(isUrlFiltered(item))
        result.append( item ); // works also for STREAM
    } 
    else {
      //! Recursive !!
      for (int i = 0; i < item->childCount(); i++) {
        result.append( getItemChildrenUrls(item->child(i)) );
      }
    }
    return result;
}
     
//! ----------------------- Favorite status method -----------------------------
bool PlsFileModel::itemIsFavorite(const MediaItemPtr item) 
{
  switch (item->type) {
    case TYPE_TRACK  : break; //! unexpected case
    case TYPE_STREAM : return item->data.at(STREAM::isFavorite).toBool();break;
    case TYPE_ARTIST : return item->data.at(ARTIST::isFavorite).toBool();break;
    case TYPE_ALBUM  : return item->data.at(ALBUM::isFavorite).toBool();break;
    case TYPE_PLAYLIST : return item->data.at(PLAYLIST::isFavorite).toBool();break;
    default: break;
  }
  return false;
}


void PlsFileModel::setItemIsFavorite(const MediaItemPtr item, bool isfavorite) 
{
  switch (item->type) {
    case TYPE_TRACK  : break; //! unexpected case  
    case TYPE_STREAM : item->data[STREAM::isFavorite] = isfavorite;break;
    case TYPE_ARTIST : item->data[ARTIST::isFavorite] = isfavorite;break;
    case TYPE_ALBUM  : item->data[ALBUM::isFavorite] = isfavorite;break;
    case TYPE_PLAYLIST : item->data[PLAYLIST::isFavorite] = isfavorite;break;
    default: break;
  }
}


//! ----------------------- playing status method ------------------------------
bool PlsFileModel::itemIsPlaying(const MediaItemPtr item)
{
  switch (item->type) {
    case TYPE_TRACK  : return item->data.at(TRACK::isPlaying).toBool();break;
    case TYPE_STREAM : return item->data.at(STREAM::isPlaying).toBool();break;
    case TYPE_ARTIST : return item->data.at(ARTIST::isPlaying).toBool();break;
    case TYPE_ALBUM  : return item->data.at(ALBUM::isPlaying).toBool();break;
    case TYPE_PLAYLIST : return item->data.at(PLAYLIST::isPlaying).toBool();break;
    default: break;
  }
  return false;
}


void PlsFileModel::setItemIsPlaying(const MediaItemPtr item, bool isplaying) 
{
  switch (item->type) {
    case TYPE_TRACK  : item->data[TRACK::isPlaying] = isplaying;break;
    case TYPE_STREAM : item->data[STREAM::isPlaying] = isplaying;break;
    case TYPE_ARTIST : item->data[ARTIST::isPlaying] = isplaying;break;
    case TYPE_ALBUM  : item->data[ALBUM::isPlaying] = isplaying;break;
    case TYPE_PLAYLIST : item->data[PLAYLIST::isPlaying] = isplaying;break;
    default: break;
  }
  
  emit signalPlayingStatusChanged();
}


MediaItemPtr PlsFileModel::getPlayingUrlItem() 
{
   for ( int i = 0; i < m_rootItem->childCount(); i++ )
   {
     MediaItemPtr playlistItem = m_rootItem->child(i);
     for (int j = 0; j < playlistItem->childCount(); j++)
     {
       if( itemIsPlaying(playlistItem->child(j)) )
         return playlistItem->child(j);
     } // end for playlist item
  } // end for playlist

  return MediaItemPtr(0);
}



void PlsFileModel::updatePlayingItem(const QString& url)
{
  for ( int i = 0; i < m_rootItem->childCount(); i++ )
   {
     MediaItemPtr playlistItem = m_rootItem->child(i);
     for (int j = 0; j < playlistItem->childCount(); j++)
     {
       MediaItemPtr urlItem = playlistItem->child(j);
       if (url == urlItem->data.at(TRACK::url).toString()) //! work for STREAM & TRACK
       {
         if(urlItem->type == TYPE_TRACK)
           urlItem->data[TRACK::isPlaying] = true;
         else
           urlItem->data[STREAM::isPlaying] = true;

         playlistItem->data[PLAYLIST::isPlaying] = true;
         //Debug::debug() << " ---- PlsFileModel::updatePlayingItem emit signalPlayingStatusChanged";
         emit signalPlayingStatusChanged();
         return;
       }
    } // end for track
  }  // end for playlist
}

void PlsFileModel::updateBrokenItem(const QString& url)
{
  for ( int i = 0; i < m_rootItem->childCount(); i++ )
  {
     MediaItemPtr playlistItem = m_rootItem->child(i);
     for (int j = 0; j < playlistItem->childCount(); j++)
     {
       MediaItemPtr trackItem = playlistItem->child(j);
       if (url == trackItem->data.at(TRACK::url).toString())
       {
         trackItem->data[TRACK::isBroken] = true;
         emit signalPlayingStatusChanged();
         return;
       }
     } // end track loop
  }  // end for playlist loop
}

void PlsFileModel::stopPlayingItem()
{
    MediaItemPtr trackItem = getPlayingUrlItem();
    if(!trackItem) return;

    if(trackItem->type == TYPE_TRACK)
      trackItem->data[TRACK::isPlaying] = false;
    else if(trackItem->type == TYPE_STREAM)
      trackItem->data[STREAM::isPlaying] = false;
    
    //! update playlist parents
    MediaItemPtr playlistItem = trackItem->parent();
    playlistItem->data[PLAYLIST::isPlaying] = false;
 
    emit signalPlayingStatusChanged();
}


//! ------------------------- filtering method ---------------------------------
// 0.0.43 : ajout return true if filterPatter = ""
bool PlsFileModel::isPlaylistFiltered(const MediaItemPtr playlistItem) 
{
  if (!playlistItem) return false;
  if (m_filter_pattern.isEmpty()) return true;
      
  //! check playlist name
  if (playlistItem->data.at(PLAYLIST::url).toString().contains(m_filter_pattern, Qt::CaseInsensitive) ||
      playlistItem->data.at(PLAYLIST::name).toString().contains(m_filter_pattern, Qt::CaseInsensitive)
     )
    return true;

  //! check playlist child (url name)
  for (int i = 0; i < playlistItem->childCount(); i++) 
  {
      MediaItemPtr trackItem = playlistItem->child(i);
      if ( (trackItem->type == TYPE_TRACK && trackItem->data.at(TRACK::title).toString().contains(m_filter_pattern, Qt::CaseInsensitive)) ||
	   (trackItem->type == TYPE_STREAM && trackItem->data.at(STREAM::name).toString().contains(m_filter_pattern, Qt::CaseInsensitive)) )
        return true;
   }

  return false;
}



bool PlsFileModel::isUrlFiltered(const MediaItemPtr urlItem) 
{
  if (!urlItem) return false;
  if (m_filter_pattern.isEmpty()) return true;
      
  //! check track item 
  if ( (urlItem->type == TYPE_TRACK && urlItem->data.at(TRACK::title).toString().contains(m_filter_pattern, Qt::CaseInsensitive)) ||
       (urlItem->type == TYPE_STREAM && urlItem->data.at(STREAM::name).toString().contains(m_filter_pattern, Qt::CaseInsensitive)) )
    return true;
    
  //! check playlist parent name & url
  MediaItemPtr playlistItem = urlItem->parent();
  if (playlistItem->data.at(PLAYLIST::url).toString().contains(m_filter_pattern, Qt::CaseInsensitive) ||
      playlistItem->data.at(PLAYLIST::name).toString().contains(m_filter_pattern, Qt::CaseInsensitive)
     )
    return true;
 
  return false;
}
