/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _COLLECTION_SCENE_H_
#define _COLLECTION_SCENE_H_

//! local
#include "CollectionView/graphicitem.h"
#include "CollectionView/graphicitem_menu.h"
#include "viewmode.h"

// Qt
#include <QStringList>
#include <QGraphicsScene>
#include <QtGui>

class CollectionModel;
class PlsFileModel;
class HistoModel;
class VirtualPlayqueue;
class PlaylistModel;

/*
********************************************************************************
*                                                                              *
*    Class CollectionScene                                                     *
*                                                                              *
********************************************************************************
*/ 
class CollectionScene : public QGraphicsScene
{
Q_OBJECT
  public: 
    CollectionScene(QWidget *parent);
    ~CollectionScene();
    void  setVirtualPlayqueue(VirtualPlayqueue  *queue);
    int   getSceneInfoSize() {return _infoSize;}

    MAIN::ViewMode                _collectionMode;

    QList<CategorieGraphicItem*> _listCategorie;    
    
    QPoint get_item_position(MediaItemPtr media); 

  private :
    CollectionModel              *_collectionModel;
    PlsFileModel                 *_plsFileModel;
    HistoModel                   *_histoModel;
  
    PlaylistModel                *_playlistModel;
    VirtualPlayqueue            *_virtualPlaylist;
    QWidget                     *_parentWidget;
        
    QList<TrackGraphicItem*>    _listTrack;    
    int                         _infoSize;

    QGraphicsItem               *_mouseGrabbedItem;
    GraphicsItemMenu            *m_graphic_item_menu;
    
  protected:
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);

  private slots :
    void slot_item_doubleclick();    
    void slot_contextmenu_triggered(ENUM_ACTION_ITEM_MENU);  

    void slot_create_new_smart_playlist();
    void slot_create_new_playlist();
    void slot_delete_playlist_editor();
    
    void slot_change_album_view();
    void slot_change_playlist_view();

    void slot_clear_history(); 
        
  private :
    //! population scene method
    void populateAlbumScene();
    void populateAlbumGridScene();
    void populateArtistScene();
    void populateTrackScene();
    void populateGenreScene();
    void populateYearScene();
    void populateFavoriteScene();
    void populatePlaylistOverviewScene();
    void populatePlaylistByTrackScene();
    void populatePlaylistSmartScene();
    void populateHistoScene();
    void populateDashBoardScene();
    void populateMostPlayedAlbum(int YPos);
    void populateTopRatedAlbum(int YPos);
    void populateMostPlayedArtist(int YPos);
    void populateTopRatedArtist(int YPos);    

    //! playing item scene method
    void playAlbum();
    void playAlbumGenre();
    void playArtist();
    void playTrack();
    void playPlaylist();

    //! playqueue method     
    void enqueueAlbum(bool is_new_playqueue);
    void enqueueAlbumGenre(bool is_new_playqueue);
    void enqueueArtist(bool is_new_playqueue);
    void enqueuePlaylist(bool is_new_playqueue);
    void enqueueTrack(bool is_new_playqueue);
            
    //! album graphic item method
    void loadCoverArtFromFile();
    void loadLastFmCover();
    void removeCoverArt();
    
    //! edition dialog
    void edit_media_dialog();
    void edit_playlist_dialog();
    
    void updateAlbumFavorite();
    void updateArtistFavorite();
    void removePlaylistFromDisk();
    void updatePlaylistFavorite();
    
  public slots:
    void populateScene();
    void updateTrackPlaying(const QString& filename);
    void updateTrackBroken(const QString& filename);
    
  signals:
    void userDatabaseUpdate(int change_on_model); // 1 = collection_model, 2 = playlist_model
};

#endif // _COLLECTION_SCENE_H_
