/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _DISCOGS_PROVIDER_H_
#define _DISCOGS_PROVIDER_H_

#include <QByteArray>
#include <QString>
#include <QPixmap>
#include <QObject>

#include "provider_base.h"
#include "networkaccess.h"


namespace DISCOGS {
/*
********************************************************************************
*                                                                              *
*    Structures Track / Release / Artist                                       *
*                                                                              *
********************************************************************************
*/
struct Track
{
    QString  m_title;
    int      m_length;
    int      m_position;
};

struct Release
{
    int                m_id;
    QString            m_type;    // master or release
    QString            m_title;
    QString            m_year;

    QString            m_thumb_url;    
    QList<Track>       m_tracks;   // list of track for release
};

struct Artist
{
    int                m_id;  
    QString            m_name;
    QString            m_discogs_url;
    QList<Release>     m_releases;
};

} // end namespace DISCOGS



/*
********************************************************************************
*                                                                              *
*    Class Discogs                                                             *
*                                                                              *
********************************************************************************
*/
class DiscogsProvider : public QObject, public InfoProvider
{
Q_OBJECT  
  public :
    DiscogsProvider(QObject* parent=0);
    ~DiscogsProvider();
    
    void init();
    void cancel_searches();
    void fetch_info(int id);
    void fetch_artist(int id, const QString &artist_name);
    void fetch_release(int id, const QString &release_name);
        
  private slots:
    void slot_parse_artist_response(QByteArray);
    void slot_artist_search_stop();
    void slot_cover_received(QByteArray);
     
  private:
    DISCOGS::Artist      m_disco_artist;
    
    int                  m_max_release;
    
    /* <QObject*, request id > */
    QMap<QObject*, int>  m_list_requests;
    /* <QObject*, release_id > */
    QMap<QObject*, int > m_map_request_album;
    
  signals:
    void artist_data_received(int,const DISCOGS::Artist& );
    //void release_data_received(int,const DISCOGS::Release& );  TO BE DONE
    void release_cover_received(int,int,QByteArray );        
    void search_artist_finished(int);
};

#endif // _DISCOGS_PROVIDER_H_
