/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "provider_echonest.h"
#include "networkaccess.h"
#include "debug.h"

// QJson
#include <qjson/parser.h>

/*
********************************************************************************
*                                                                              *
*    Class EchonestProvider                                                    *
*                                                                              *
********************************************************************************
*/
EchonestProvider::EchonestProvider(QObject* parent)  : InfoProvider("echonest")
{
}

EchonestProvider::~EchonestProvider()
{
    cancel_searches();
}

/*******************************************************************************
  EchonestProvider::init
*******************************************************************************/ 
void EchonestProvider::init() 
{
}

/*******************************************************************************
  EchonestProvider::cancel_searches
*******************************************************************************/ 
void EchonestProvider::cancel_searches() 
{
   /* Debug::debug() << "EchonestProvider: cancel_searches";
    foreach (QObject* object, m_requests.keys()) {
      m_requests.take(object);
      delete object;
    }
      
    m_requests.clear();
    m_echonest_artists.clear();*/
}


/*******************************************************************************
  EchonestProvider::fetch_info
*******************************************************************************/ 
void EchonestProvider::fetch_info(int id) 
{
    if( !is_enabled() ) return;

    Debug::debug() << "EchonestProvider: fetch_info id :" << id;
    
    fetch_artist_image(id);
    fetch_artist_bio(id);
    fetch_artist_similar(id);
}


/*******************************************************************************
  EchonestProvider::fetch_artist_bio
*******************************************************************************/
void EchonestProvider::fetch_artist_bio(int id)
{
    Debug::debug() << " ---- EchonestProvider::fetchArtistInfo ARTIST" << m_current_media->data[TRACK::artist].toString();

    Echonest::Artist* echonest_artist = new Echonest::Artist();
    echonest_artist->setName(m_current_media->data.at(TRACK::artist).toString());

    QNetworkReply* reply = echonest_artist->fetchBiographies();
    m_requests[reply] = id;
    m_echonest_artists[reply] = echonest_artist;

    connect(reply, SIGNAL(finished()), SLOT(slot_fetch_artist_bio_done()));
}

/*******************************************************************************
  EchonestProvider::slot_fetch_artist_bio_done
*******************************************************************************/
void EchonestProvider::slot_fetch_artist_bio_done() 
{
    Debug::debug() << " ---- EchonestProvider::slot_fetch_artist_bio_done";
    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
    if (!reply || !m_requests.contains(reply))   return;

    const int id = m_requests.take(reply);
    reply->deleteLater();

    Echonest::Artist* echonest_artist = m_echonest_artists.take(reply);

    try {
      echonest_artist->parseProfile(reply);
    } catch (Echonest::ParseError e) {
      qWarning() << "slot_fetch_artist_bio_done Error parsing echonest reply:" << e.errorType() << e.what();
    }

    //! search for last fm
    foreach (const Echonest::Biography& bio, echonest_artist->biographies()) {
      QString canonical_site = bio.site().toLower();
      canonical_site.replace(QRegExp("[^a-z]"),"");
      //Debug::debug() << " ---- EchonestProvider::RequestFinished canonical_site" << canonical_site;
      if(canonical_site == "lastfm") {
        emit ArtistBioOK(id,canonical_site,bio.text());
        return;
      }
    }  

    //! search for wikipedia first
    foreach (const Echonest::Biography& bio, echonest_artist->biographies()) {
      QString canonical_site = bio.site().toLower();
      canonical_site.replace(QRegExp("[^a-z]"),"");
      //Debug::debug() << " ---- EchonestProvider::RequestFinished canonical_site" << canonical_site;
      if(canonical_site == "wikipedia") {
        emit ArtistBioOK(id,canonical_site,bio.text());
        return;
      }
    }
    
    //! search for others
    foreach (const Echonest::Biography& bio, echonest_artist->biographies()) {
      QString canonical_site = bio.site().toLower();
      canonical_site.replace(QRegExp("[^a-z]"),"");
      //Debug::debug() << " ---- EchonestProvider::RequestFinished canonical_site" << canonical_site;
      emit ArtistBioOK(id,canonical_site,bio.text());
      return;
    }
    
    //delete echonest_artist;  
}



/*******************************************************************************
  EchonestProvider::slot_fetch_artist_bio_done
*******************************************************************************/
void EchonestProvider::fetch_artist_image(int id)
{
    Debug::debug() << " ---- EchonestProvider::fetch_artist_image ARTIST" << m_current_media->data[TRACK::artist].toString();

    Echonest::Artist* echonest_artist = new Echonest::Artist();
    echonest_artist->setName(m_current_media->data.at(TRACK::artist).toString());

    QNetworkReply* reply = echonest_artist->fetchImages();
    m_requests[reply] = id;
    m_echonest_artists[reply] = echonest_artist;

    connect(reply, SIGNAL(finished()), SLOT(slot_fetch_artist_image_done()));
}



/*******************************************************************************
  EchonestProvider::slot_fetch_artist_image_done
*******************************************************************************/
void EchonestProvider::slot_fetch_artist_image_done()
{
    Debug::debug() << " ---- EchonestProvider::RequestImageFinished";
    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
    if (!reply || !m_requests.contains(reply))   return;

    const int id = m_requests.take(reply);
    reply->deleteLater();

    Echonest::Artist* echonest_artist = m_echonest_artists.take(reply);

    try {
      echonest_artist->parseProfile(reply);
    } catch (Echonest::ParseError e) {
      qWarning() << "Error parsing echonest reply:" << e.errorType() << e.what();
    }

    foreach (const Echonest::ArtistImage& image, echonest_artist->images()) {
      //Debug::debug() << " ---- EchonestProvider::RequestImageFinished image.url()" << image.url();
      QObject* reply = HTTP()->get(image.url());
      m_requests[reply] = id;
      m_echonest_artists[reply] = echonest_artist;            
      
      connect(reply, SIGNAL(data(QByteArray)), SLOT(slot_artist_image_fetched(QByteArray)));
      // break at first image found  
      break;
    }  
}

/*******************************************************************************
  EchonestProvider::slot_artist_image_fetched
*******************************************************************************/
void EchonestProvider::slot_artist_image_fetched(QByteArray bytes)
{
    Debug::debug() << " ---- EchonestProvider::slot_artist_image_fetched";

    QObject* reply = qobject_cast<QObject*>(sender());
    if (!reply || !m_requests.contains(reply))   return;
    
    const int id = m_requests.take(reply);

    emit ArtistImageOK(id,bytes);
    
    reply->deleteLater();
    m_echonest_artists.take(reply);
}



/*******************************************************************************
  EchonestProvider::fetch_artist_similar
*******************************************************************************/
void EchonestProvider::fetch_artist_similar(int id)
{
    Debug::debug() << " ---- EchonestProvider::fetch_artist_similar";
    using Echonest::Artist;

    Artist::SearchParams params;
    params << Artist::SearchParamEntry(Artist::Name, m_current_media->data.at(TRACK::artist).toString());
    params << Artist::SearchParamEntry(Artist::MinHotttnesss, 0.5);

    QNetworkReply* reply = Echonest::Artist::fetchSimilar(params);
    connect(reply, SIGNAL(finished()), SLOT(slot_fetch_artist_similar_done()));
    m_requests[reply] = id;
}


/*******************************************************************************
  EchonestProvider::slot_fetch_artist_similar_done
*******************************************************************************/
void EchonestProvider::slot_fetch_artist_similar_done()
{
    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
    if (!reply || !m_requests.contains(reply))  return;
    
    reply->deleteLater();

    int id = m_requests.take(reply);

    Echonest::Artists artists;
    try {
       artists = Echonest::Artist::parseSimilar(reply);
    } catch (Echonest::ParseError e) {
       qWarning() << "Error parsing echonest reply:" << e.errorType() << e.what();
    }

    QStringList similar;
    if (!artists.isEmpty()) {
      foreach (const Echonest::Artist& artist, artists) {
          similar << artist.name();
      }
      emit ArtistSimilarOK(id,similar); 
    }
}