/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//! LOCAL
#include "provider_lyrics.h"
#include "ultimatelyricsprovider.h"

#include "networkaccess.h"
#include "utilities.h"
#include "constants.h"

#include "debug.h"
/*
********************************************************************************
*                                                                              *
*    Class LyricsProvider                                                      *
*                                                                              *
********************************************************************************
*/ 
LyricsProvider::LyricsProvider(QObject* parent) : InfoProvider("lyrics")
{
}

LyricsProvider::~LyricsProvider()
{
    cancel_searches();
}
/*******************************************************************************
  LyricsProvider::cancel_searches
*******************************************************************************/ 
void LyricsProvider::cancel_searches() 
{
    //Debug::debug() << "LyricsProvider: cancel_searches";
    // to be done cancel pending ultimate lyrics providers ?
}

/*******************************************************************************
  LyricsProvider::fetch_info
*******************************************************************************/ 
void LyricsProvider::fetch_info(int id)
{
    if( !is_enabled() ) return;
    
    m_current_providers.clear();
    m_current_providers.append(m_providers);
    
    if( !is_local_lyrics(id) )
      slot_fetch_lyrics(id);
}
/*******************************************************************************
  LyricsProvider::init
*******************************************************************************/
void LyricsProvider::init()
{
    //Debug::debug() << " ---- LyricsProvider::init";
    m_ultimateLyricsReader = new UltimateLyricsReader(this);
  
    QFuture<ProviderList> future = QtConcurrent::run(
             m_ultimateLyricsReader, &UltimateLyricsReader::Parse,
             QString(":/data/lyrics/ultimate_providers.xml")
     );
  
    QFutureWatcher<ProviderList>* watcher = new QFutureWatcher<ProviderList>(this);
    watcher->setFuture(future);
    connect(watcher, SIGNAL(finished()), SLOT(slot_ultimate_lyrics_parsed()));
}

/*******************************************************************************
  LyricsProvider::slot_ultimate_lyrics_parsed
*******************************************************************************/
void LyricsProvider::slot_ultimate_lyrics_parsed() 
{
    //Debug::debug() << " ---- LyricsProvider::slot_ultimate_lyrics_parsed";  
    ProviderList list;

    QFutureWatcher<ProviderList>* watcher = static_cast<QFutureWatcher<ProviderList>*>(sender());

    foreach (UltimateLyricsProvider* provider, watcher->result()) {
      //Debug::debug() << " ---- LyricsProvider::slot_ultimate_lyrics_parsed Provider Name =" << provider->name();
      list << provider;
    }
  
    //! sort lyrics providers 
    QStringList default_order;
    default_order << "lyrics.wikia.com"
                  << "lyricstime.com"
                  << "lyricsreg.com"
                  << "lyricsmania.com"
                  << "metrolyrics.com"
                  << "azlyrics.com"
                  << "songlyrics.com"
                  << "elyrics.net"
                  << "lyricsdownload.com"
                  << "lyrics.com"
                  << "lyricsbay.com"
                  << "directlyrics.com"
                  << "loudson.gs"
                  << "teksty.org";/*
                  << "tekstowo.pl (Polish translations)"
                  << "vagalume.uol.com.br"
                  << "vagalume.uol.com.br (Portuguese translations)"
                  << "darklyrics.com";*/
  
    foreach (const QString& name, default_order) {
      foreach (UltimateLyricsProvider* provider, list) {
        if (provider->name() == name) {
          Debug::debug() << " ---- LyricsProvider::slot_ultimate_lyrics_parsed Provider Name =" << provider->name();
          m_providers << provider;
        }
      }
    }
  
    watcher->deleteLater();
    delete m_ultimateLyricsReader;
}

/*******************************************************************************
  LyricsProvider::is_local_lyrics
*******************************************************************************/
bool LyricsProvider::is_local_lyrics(int id)
{
    //! Build Lyrics Hash (Artist Name + Song Name)
    QCryptographicHash hash(QCryptographicHash::Sha1);
    hash.addData(m_current_media->data.at(TRACK::artist).toString().toLower().toUtf8().constData());
    hash.addData(m_current_media->data.at(TRACK::title).toString().toLower().toUtf8().constData());

    const QString lyrics_file_path = QString(UTIL::CONFIGDIR + "/lyrics/" + hash.result().toHex() + ".txt");
    //Debug::debug() << " ---- LyricsProvider::searchLocalLyricsInfo" << lyrics_file_path;

    if( QFile::exists(lyrics_file_path) )
    {
      QFile file(lyrics_file_path);
    
      if(file.open(QIODevice::ReadOnly)) {
        QString file_content = QTextStream( &file ).readAll();
        file_content.replace(QRegExp("\n"), "<br>");

        LYRICS::Result result;
        result.m_provider_name = QString("local");
        result.m_lyrics        = file_content;
        /*result.m_url           = lyrics_file_path;*/
      
        emit LyricsOK(id,result); 
        return true;
      }
    }
        
    return false;
}

/*******************************************************************************
  LyricsProvider::slot_fetch_lyrics
*******************************************************************************/
void LyricsProvider::slot_fetch_lyrics(int id)
{
    if(m_current_providers.isEmpty()) return;
  
    UltimateLyricsProvider *provider = m_current_providers.takeFirst();
    provider->FetchInfo(id, m_current_media);
  
    Debug::debug() << "LyricsProvider::slot_fetch_lyrics for " << provider->name();
    
    connect(provider, SIGNAL(InfoReady(int, const QString&)), this, SLOT(slot_lyrics_fetched(int,const QString&)), Qt::QueuedConnection);
    connect(provider, SIGNAL(Finished(int)), this, SLOT(slot_fetch_lyrics(int)), Qt::QueuedConnection);
}


/*******************************************************************************
  LyricsProvider::slot_lyrics_fethed
*******************************************************************************/
void LyricsProvider::slot_lyrics_fetched(int id,const QString& lyric)
{
    // get provider sender
    UltimateLyricsProvider* provider = qobject_cast<UltimateLyricsProvider*>(sender());
    
    if(provider) {
      LYRICS::Result result;
    
      result.m_provider_name = provider->name();
      result.m_lyrics        = lyric;
      result.m_url           = provider->lyricsUrl();
      
      emit LyricsOK(id,result); 
    }
}


