/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Dialog/aboutdialog.h"
#include "constants.h"

#include <QtGui>
#include <QtCore>

/*
********************************************************************************
*                                                                              *
*    Class AboutDialog                                                         *
*                                                                              *
********************************************************************************
*/
AboutDialog::AboutDialog(QWidget *parent) : QDialog(parent)
{   
    this->setWindowTitle(tr("About Yarock"));
    this->resize(400, 450);

    QVBoxLayout* layout = new QVBoxLayout();

    QFrame* header = new QFrame();
    header->setFrameShape(QFrame::StyledPanel);
    header->setFrameShadow(QFrame::Sunken);
    
    QHBoxLayout * headerLayout = new QHBoxLayout();
    QLabel *icon = new QLabel(this);
    
    icon->setPixmap( QPixmap(":/icon/64x64/yarock.png") );
    
    headerLayout->addWidget(icon);
    headerLayout->addWidget(
       new QLabel(QString("<h2>Yarock</h2><p><b>Version:</b> %1</p>").arg(VERSION))
    );
    
    headerLayout->addStretch();

    header->setLayout(headerLayout);

    QTabWidget* tabs = new QTabWidget();

    QTextBrowser* aboutTab = new QTextBrowser();
    aboutTab->setFrameShape(QFrame::NoFrame);
    aboutTab->setStyleSheet("QTextBrowser { background:transparent }");
    aboutTab->setOpenExternalLinks(true);
    aboutTab->setHtml("<p><b>A Moder Music player</b></p>"
                      ""
                      "<p>&copy;2010-2011 by Sebastien Amardeilh<br/>"
                      "<a href=\"http://qt-apps.org/content/show.php/YaRock?content=129372\">yarock on Qt app</a><br/>"
                      "<a href=\"https://launchpad.net/yarock\">yarock on lauchpad</a></p>"
                      ""
                      "<p>Yarock is licensed under the <a href=\"http://www.gnu.org/licenses/gpl-3.0.txt\">GPLv3</a> license.</p>"
                      "<p>Please use <a href=\"https://bugs.launchpad.net/yarock\">https://bugs.launchpad.net/yarock</a> to report bugs.</p>"
                      ""
                      "<p>Please use <a href=\"https://www.transifex.net/projects/p/yarock-translation/\">yarock translations</a> for translations.</p>"
                       );

    QTextBrowser* authorsTab = new QTextBrowser();
    authorsTab->setFrameShape(QFrame::NoFrame);
    authorsTab->setStyleSheet("QTextBrowser { background:transparent }");
    authorsTab->setOpenExternalLinks(true);
    authorsTab->setHtml("Yarock is developed by:"
                        "<dl>"
                        "<dt><b>Sebastien Amardeilh</b></dt>"
                        "<dd><a href=\"mailto:sebastien.amardeilh+yarock@gmail.com\">sebastien.amardeilh+yarock@gmail.com</a><br/>"
                        "</dl>");
                           
    QTextBrowser* thanksTab = new QTextBrowser();
    thanksTab->setFrameShape(QFrame::NoFrame);
    thanksTab->setStyleSheet("QTextBrowser { background:transparent }");
    thanksTab->setOpenExternalLinks(true);
    thanksTab->setHtml("<b>Special thanks go to:</b><br/>"
                       "<dl>"
                       "<dt>Pavel Fric (for Czech translation)</bt>"
                       "<dl>"
                       "<dl>"
                       "<b>Help translating</b>"
                       "<p>use <a href=\"https://www.transifex.net/projects/p/yarock-translation/\">yarock translations</a> for translations.</p>"
                      );
                          

    tabs->addTab(aboutTab, "About");
    tabs->addTab(authorsTab, "Authors");
    tabs->addTab(thanksTab, "Thanks to");

    QHBoxLayout* closeLayout = new QHBoxLayout();
    closeLayout->addStretch();
    QPushButton* closeButton = new QPushButton("Close");

    QObject::connect(closeButton, SIGNAL(clicked()), this, SLOT(accept()) );
     
    closeLayout->addWidget(closeButton);

    layout->addWidget(header);
    layout->addWidget(tabs);
    layout->addLayout(closeLayout);

    this->setLayout(layout);
}
