/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "playlistmodel.h"
#include "MediaItem/mediaitem.h"
#include "MediaItem/mediamimedata.h"
//#include "debug.h"

bool sortByModelIndexRow(MediaItemPtr a,MediaItemPtr b) 
{
  return (PlaylistModel::instance()->rowForMediaItem(a) 
         <
         PlaylistModel::instance()->rowForMediaItem(b));
}




PlaylistModel* PlaylistModel::INSTANCE = 0;
/*
********************************************************************************
*                                                                              *
*    Class PlaylistModel                                                       *
*                                                                              *
********************************************************************************
*/
PlaylistModel::PlaylistModel(QWidget *parent) : QAbstractListModel(parent) 
{
    INSTANCE             = this;
    _playingMediaItem    =  MediaItemPtr(0);
    _requestedMediaItem  =  MediaItemPtr(0);
    m_stop_after_media   =  MediaItemPtr(0);
}

void PlaylistModel::needUpdate()
{
    emit layoutChanged(); 
}

//!------------PlaylistModel::clear --------------------------------------------
void PlaylistModel::clear()
{
    foreach(MediaItemPtr mi, _listMediaItem)
      mi.reset();
    
    _listMediaItem.clear();
    _playingMediaItem     =  MediaItemPtr(0);
    _requestedMediaItem   =  MediaItemPtr(0);
  
    emit layoutChanged();
    emit dataChanged(QModelIndex(), QModelIndex());  
}

/*******************************************************************************
    QAbstractListModel implementation
*******************************************************************************/
int PlaylistModel::rowCount(const QModelIndex & /* parent */) const 
{
    return _listMediaItem.size();
}

QVariant PlaylistModel::data(const QModelIndex &index, int role) const
{
Q_UNUSED(index)
Q_UNUSED(role)

    return QVariant();
}

int PlaylistModel::rowForMediaItem(const MediaItemPtr mi)
{
    return _listMediaItem.indexOf(mi);
}


//!------------PlaylistModel::updateMediaItemBroken ----------------------------
void PlaylistModel::updateMediaItemBroken(const QString & filename)
{
    //Debug::debug() << " --- PlaylistModel->update broken media item" << filename;
    if(_requestedMediaItem) {
      if(MEDIA::urlForMediaItem(_requestedMediaItem) == filename) {
        _requestedMediaItem->data[TRACK::isBroken]  = true;
        _requestedMediaItem->data[TRACK::isPlaying] = false;

        int row = _listMediaItem.indexOf(_requestedMediaItem);
        QModelIndex newIndex = index(row, 0, QModelIndex());
        emit dataChanged(newIndex, newIndex);   
        return;
      }
    }

    if(_playingMediaItem) {
      if(MEDIA::urlForMediaItem(_playingMediaItem) == filename) {
        _playingMediaItem->data[TRACK::isBroken]  = true;
        _playingMediaItem->data[TRACK::isPlaying] = false;

        int row = _listMediaItem.indexOf(_playingMediaItem);
        QModelIndex newIndex = index(row, 0, QModelIndex());
        emit dataChanged(newIndex, newIndex);  
        return;
      }
    }

    foreach(MediaItemPtr mi, _listMediaItem) {
      if(MEDIA::urlForMediaItem(mi) == filename) {
        mi->data[TRACK::isBroken]  = true;
        mi->data[TRACK::isPlaying] = false;    
      
        int row = _listMediaItem.indexOf(mi);
        QModelIndex newIndex = index(row, 0, QModelIndex());
        emit dataChanged(newIndex, newIndex);  
        break;
      }
    }
}


//!------------PlaylistModel::updateMediaItemPlaying----------------------------
// 7-12-2012 ; correction de la comparaison des ulr avec le filename
// dans le cas de sliens symboliques la fonction MEDIA::urlForMediaItem
// en renvoyait pas la meme url que celle de phonon (filename)
void PlaylistModel::updateMediaItemPlaying(const QString & filename)
{
    //Debug::debug() << " --- PlaylistModel->update playing media item";

    //! unset previous playing item
    stopPlayingMediaItem();

    //! set new playing item
    if(_requestedMediaItem) {
      if(_requestedMediaItem->data.at(TRACK::url).toString() == filename) {
        _playingMediaItem = _requestedMediaItem;
        _playingMediaItem->data[TRACK::isPlaying] = true;
        _playingMediaItem->data[TRACK::isPlayed]  = true;

        int row = _listMediaItem.indexOf(_playingMediaItem);
        QModelIndex newIndex = index(row, 0, QModelIndex());
        emit dataChanged(newIndex, newIndex);   
        return;
      }
    }

    foreach(MediaItemPtr mi, _listMediaItem) {
      if(mi->data.at(TRACK::url).toString() == filename) {
        _playingMediaItem = mi;
        _playingMediaItem->data[TRACK::isPlaying] = true;
        _playingMediaItem->data[TRACK::isPlayed]  = true;

        int row = _listMediaItem.indexOf(_playingMediaItem);
        QModelIndex newIndex = index(row, 0, QModelIndex());
        emit dataChanged(newIndex, newIndex);   
        break;
      }
    }
}

//!------------PlaylistModel::stopPlayingMediaItem -----------------------------
void PlaylistModel::stopPlayingMediaItem()
{
    //! unset previous playing item
    if(_playingMediaItem) {
      _playingMediaItem->data[TRACK::isPlaying] = false;
      
      int row = _listMediaItem.indexOf(_playingMediaItem);
      QModelIndex newIndex = index(row, 0, QModelIndex());
      emit dataChanged(newIndex, newIndex);   

      _playingMediaItem =  MediaItemPtr(0);
    }
  
}



//!------------PlaylistModel::unplayedMedia ------------------------------------
QList<MediaItemPtr> PlaylistModel::unplayedMediaItem()
{
    //Debug::debug() << "--- PlaylistModel::unplayedMediaItem";

    QList<MediaItemPtr>  list;
    foreach (MediaItemPtr mi, _listMediaItem) {
      if(mi->data.at(TRACK::isPlayed).toBool() == false) //! ok for STREAM 
        list << mi;
    }  
    
    return list;
}

//!------------PlaylistModel::setRequestedMediaItem ----------------------------
void PlaylistModel::setRequestedMediaItem(int row)
{
  if (!rowExists(row)) return;

  _requestedMediaItem = _listMediaItem.at(row);
}

MediaItemPtr PlaylistModel::getRequestedMediaItem() const 
{
    if(!_requestedMediaItem) 
      return MediaItemPtr();
    
    return _requestedMediaItem;
}


//!------------PlaylistModel::skipBackward -------------------------------------
// Specific method
void PlaylistModel::skipBackward()
{
   const int prevRow = rowForMediaItem(_requestedMediaItem) - 1;
   if (rowExists(prevRow))
     setRequestedMediaItem(prevRow);
}

//!------------PlaylistModel::skipForward --------------------------------------
// Specific method
void PlaylistModel::skipForward() 
{
   const int nextRow = rowForMediaItem(_requestedMediaItem) + 1;
   if (rowExists(nextRow))
       setRequestedMediaItem(nextRow);
}

//!------------PlaylistModel::MediaItemAt --------------------------------------
// Specific method
MediaItemPtr PlaylistModel::MediaItemAt(int row) const 
{
    if (rowExists(row)) {
      return _listMediaItem.at(row);
    }
    return MediaItemPtr();
}

//!------------PlaylistModel::getActiveMediaItem -------------------------------
// Specific method
MediaItemPtr PlaylistModel::getPlayingMediaItem() const
{
    if(_playingMediaItem)
      return _playingMediaItem;
    return MediaItemPtr();
}

int PlaylistModel::getPlayingRow() 
{
    if(_playingMediaItem)
      return _listMediaItem.indexOf(_playingMediaItem);
    else
      return -1;
}

/*******************************************************************************
    get_stop_after_media
    set_stop_after_media
*******************************************************************************/
MediaItemPtr PlaylistModel::get_stop_after_media() const
{
    return m_stop_after_media;
}

void PlaylistModel::set_stop_after_media(MediaItemPtr media)
{
    m_stop_after_media = media;
}

/*******************************************************************************
    addMediaItem
    addMediaItems
    insertMediaItem
    removeDuplicate
*******************************************************************************/
void PlaylistModel::addMediaItem(const MediaItemPtr media) 
{
    _mutex.lock();
    beginInsertRows(QModelIndex(), _listMediaItem.size(), _listMediaItem.size());
    _listMediaItem << MediaItemPtr(media);
    endInsertRows();
    _mutex.unlock();
     
    if(media->data.at(TRACK::isPlaying).toBool())
      _playingMediaItem = media;
             
    emit layoutChanged();
}

void PlaylistModel::addMediaItems(QList<MediaItemPtr> list) 
{
    _mutex.lock();
    beginInsertRows(QModelIndex(), _listMediaItem.size(), _listMediaItem.size());
    
    foreach (const MediaItemPtr media, list) {
      _listMediaItem << media;

      if(media->data.at(TRACK::isPlaying).toBool())
        _playingMediaItem = media;      
    }
    
    endInsertRows();
    _mutex.unlock();

             
    emit layoutChanged();
}

void PlaylistModel::insertMediaItem(const MediaItemPtr media, int pos) 
{
    //Debug::debug() << "--- PlaylistModel-->insertMediaItem media" << media;
    const int start = pos == -1 ? rowCount() : pos;    
    
    _mutex.lock();
    beginInsertRows(QModelIndex(), start, start);
    _listMediaItem.insert(start, MediaItemPtr(media));
    endInsertRows();
    _mutex.unlock();
     
    if(media->data.at(TRACK::isPlaying).toBool())
      _playingMediaItem = media;
             
    emit layoutChanged();
}

void PlaylistModel::removeDuplicate()
{
    QSet<QUrl> filenames;

    foreach(MediaItemPtr media, _listMediaItem) {
        if(filenames.contains( media->data.at(TRACK::url).toString() ) ) {
          int row = _listMediaItem.indexOf(media);
          removeRows(row,row,QModelIndex());
        }
        else {
          filenames.insert( media->data.at(TRACK::url).toString() );        
        }
    }
}

   
    
//!------------PlaylistModel::getListDuration ----------------------------------
QString PlaylistModel::getListDuration()
{ 
  return MEDIA::durationTextFromMediaList(_listMediaItem); 
}





//!------------PlaylistModel::removeRows ---------------------------------------
bool PlaylistModel::removeRows(int position, int count, const QModelIndex & /*parent*/) 
{
    //! version 0.0.43 suppression des beginRemoveRows & endRemoveRows car crash
    //Debug::debug() << "--- PlaylistModel-->removeRow position rows" << position;
    //Debug::debug() << "--- PlaylistModel-->removeRow position count" << count;
    if( position + count - 1 >= _listMediaItem.size()) return false;
    
    _mutex.lock();
    beginRemoveRows(QModelIndex(), position, position + count - 1);
    for (int row = 0; row < count; row++) {
      MediaItemPtr mi = _listMediaItem.takeAt(position);
      mi.reset();
    }
    endRemoveRows();
    _mutex.unlock();

    emit layoutChanged();
    return true;
}



/*******************************************************************************
    DRAG AND DROP
    -> supportedDropActions
    -> flags
    -> mimeTypes
    -> mimeData
    -> dropMimeData
*******************************************************************************/
Qt::DropActions PlaylistModel::supportedDropActions() const 
{
    return Qt::MoveAction | Qt::CopyAction | Qt::LinkAction;
}


Qt::ItemFlags PlaylistModel::flags(const QModelIndex &index) const
{
  Qt::ItemFlags flags = Qt::ItemIsEnabled | Qt::ItemIsSelectable;

  if (index.isValid())
    return flags | Qt::ItemIsDragEnabled;

  return Qt::ItemIsDropEnabled;  
}

QStringList PlaylistModel::mimeTypes() const
{
    // Encoding Exported Data with mimeTypes
    QStringList types;
    types << MEDIA_MIME;
    types << "text/uri-list"; //we do accept urls
    return types;
}

QMimeData* PlaylistModel::mimeData( const QModelIndexList &indexes ) const 
{
    MediaMimeData* mime = new MediaMimeData();
    mime->setSource(SOURCE_PLAYQUEUE);

    foreach( const QModelIndex &index, indexes ) {
        int row = index.row();
        
        if (row >= 0 && row < _listMediaItem.size())
            mime->addMediaItem( _listMediaItem.at(row) );
    }

    return mime;
}


//!------------PlaylistModel::dropMimeData -------------------------------------
// --> Inserting Dropped Data into a Model
bool PlaylistModel::dropMimeData(const QMimeData *data,
                                 Qt::DropAction action, int row, int column,
                                 const QModelIndex &parent) 
{
    QAbstractListModel::dropMimeData(data, action, row, column, parent);
    
    if (action == Qt::IgnoreAction) return true;
    
    if (data->hasFormat(MEDIA_MIME)) {
      //Debug::debug() << "--- PlaylistModel::dropMimeData --> data->hasFormat(MEDIA_MIME)";
      const MediaMimeData* mediaMimeData = dynamic_cast<const MediaMimeData*>( data );
      if(!mediaMimeData)
        return false;
    
      // internal move
      if(mediaMimeData->_source == SOURCE_PLAYQUEUE) {
        move(mediaMimeData, row);
      }
      // media item from collection
      else {
        m_thread_manager->playlistAddMediaItems(mediaMimeData->getMediaItem(), row);
      }
      return true;
    }
    else if (data->hasUrls()) {
      m_thread_manager->playlistAddUrls(data->urls(), row);
      return true;    
    }

    return false; 
}



//!------------PlaylistModel::move ---------------------------------------------
void PlaylistModel::move(const MediaMimeData *data, int row) 
{
    QList<MediaItemPtr> moved_items = data->getMediaItem();

    qSort(moved_items.begin(), moved_items.end(), sortByModelIndexRow);
    
    foreach (MediaItemPtr mi, moved_items) {
      int source_row = rowForMediaItem(mi);    
      _listMediaItem.takeAt(source_row);      
      if (row != -1 && row >= source_row)
        row --;
    }
    
    // Put the items back in
    const int start = row == -1 ? rowCount() : row;
    _mutex.lock();
    
    for (int i=start ; i<start + moved_items.count() ; i++) 
    {
      beginInsertRows(QModelIndex(),  i,  i);
      _listMediaItem.insert(i, MediaItemPtr(moved_items.at(i - start)));
      endInsertRows();
    }  
    _mutex.unlock();      
  
    emit needSelectionAfterMove(moved_items);
    emit layoutChanged();
}
