/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "StreamModel/icecastmanager.h"
#include "networkaccess.h"
#include "utilities.h"
#include "debug.h"

#include <QtCore>
#include <QtNetwork>


/*
********************************************************************************
*                                                                              *
*    Class IceCastManager                                                      *
*                                                                              *
********************************************************************************
*/
IceCastManager::IceCastManager() 
{
    Debug::debug() << " ---- IceCastManager --> creation";
    _isUpdating     = false;
    _isError        = false;
    _downloadTry    = 0;
    _showDuplicate  = true;
}


QList<MediaItemPtr> IceCastManager::getIceCastStream() const
{
    return readFile();
}


bool IceCastManager::isReady()
{
    //! pour savoir si il faut tlcharger ou non
    QFile file(UTIL::CONFIGDIR + "/radio/" + "icecast.xml");
    return (file.exists() && !_isUpdating);
}

bool IceCastManager::isError()
{
    return _isError && !isReady();
}



//! ------------------ startDownload -------------------------------------------  
void IceCastManager::startDownload() 
{
    Debug::debug() << " ---- IceCastManager --> startDownload";
    if(_isUpdating) return;
    
    if(_downloadTry++ > 2) {
        _isError    = true;
        _isUpdating = false;
        return;
    }
    _isUpdating = true;
    sendRequest();
}


//! ------------------  sendRequest -------------------------------------------  
void IceCastManager::sendRequest() 
{
    Debug::debug() << " ---- IceCastManager -> sendRequest";

    QUrl url("http://dir.xiph.org/yp.xml");

    QObject *reply = HTTP()->get(url);
    connect(reply, SIGNAL(data(QByteArray)), this, SLOT(parseResponse(QByteArray)));
    connect(reply, SIGNAL(error(QNetworkReply*)), this, SLOT(iceCastStop()));
}

//! ------------------  parseResponse -----------------------------------------  
void IceCastManager::parseResponse(QByteArray bytes) 
{
    QFile file(UTIL::CONFIGDIR + "/radio/icecast.xml");

    if(file.exists()) file.remove();
  
    if (!file.open(QIODevice::WriteOnly)) {
        emit iceCastStop();
        return;
    }
    // serialize the data into the file
    QDataStream stream( &file ); 
    stream.writeRawData(bytes.constData(), bytes.size());
    
    emit iceCastStop();
}

//! ------------------  readFile -----------------------------------------------  
QList<MediaItemPtr> IceCastManager::readFile() const  
{
    QList<MediaItemPtr> listStream;
      
    Debug::debug() << " ---- IceCastManager -> readFile";
    QFile file(UTIL::CONFIGDIR + "/radio/" + "icecast.xml");

    if(!file.exists()) return listStream;
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) return listStream;

    QByteArray bytes = file.readAll();
    QXmlStreamReader xml(bytes);
    MediaItemPtr mi = MediaItemPtr(0);

    while(!xml.atEnd() && !xml.hasError()) { 
      xml.readNext();
      if (xml.isStartElement()) {
          if(xml.name() == "entry") {
              //! new stream entry
              mi = MediaItemPtr(new MediaItem(TYPE_STREAM));
          } else if (xml.name() == "server_name") {
              mi->data[STREAM::name] = QString(xml.readElementText());
          } else if (xml.name() == "listen_url") {
              mi->data[STREAM::url]  = QString(xml.readElementText());
              
              // 0.0.57 les noms de stations apparaissent en "Unspecified name"
              QString server_name = mi->data[STREAM::name].toString();
              if(server_name == QString("Unspecified name") || server_name.isEmpty() ) 
              {
                  mi->data[STREAM::name] = mi->data[STREAM::url].toString();
              }              
          } else if (xml.name() == "genre") {
              mi->data[STREAM::categorie] = QString(xml.readElementText());              
          }
      }
      else if (xml.isEndElement()) {
          if(xml.name() == "entry") {
              //! if stream OK --> add it
              if (!mi->data.at(STREAM::url).toString().isEmpty() && 
                  !mi->data.at(STREAM::name).toString().isEmpty()) {
                  listStream.append(mi);
              }
          }
      }
    }
    
    if(!_showDuplicate) 
    {
      qSort(listStream.begin(), listStream.end(), MEDIA::compareStreamName);
      QString _current;
      foreach (MediaItemPtr mi, listStream) {
          QString _new = mi->data.at(STREAM::name).toString();
          if(_current == _new) {
            listStream.removeOne ( mi );
          }
          else {
            _current = _new;
          }
      }
    }
    qSort(listStream.begin(), listStream.end(), MEDIA::compareStreamCategorie);


    return listStream;
}

//! ------------------  iceCastStop --------------------------------------------  
void IceCastManager::iceCastStop()  
{
    Debug::debug() << " ---- IceCastManager -> iceCastStop";
    _isUpdating = false;
    emit iceCastUpdated();
}
