/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef _SHOUTCAST_MANAGER_H_
#define _SHOUTCAST_MANAGER_H_

#include "MediaItem/mediaitem.h"

#include <QList>
#include <QString>
#include <QStringList>
#include <QByteArray>
#include <QObject>

/*
********************************************************************************
*                                                                              *
*    Class ShoutCastManager                                                    *
*                                                                              *
********************************************************************************
*/
class ShoutCastManager : public QObject
{
Q_OBJECT
  public:
    ShoutCastManager();
    QList<MediaItemPtr> getShoutCastStream() const;
    bool isReady();
    bool isError();
    void startDownload();
    void setDuplicate(bool b) {_showDuplicate = b;}

  private:
    bool _isUpdating;
    bool _isError;
    int  _downloadTry;
    QStringList        _shoutGenreList;
    QList<MediaItemPtr>  _stationList; 
    bool _showDuplicate;
        
  private slots:
    void sendGenreRequest();
    void parseGenreResponse(QByteArray bytes);
    
    void sendStationRequest(QString genre);
    void parseStationResponse(QByteArray bytes, QString &genre);
        
    void writeFile(const QList<MediaItemPtr> &liste);
    QList<MediaItemPtr> readFile() const;
    
    void shoutCastStop();
    void shoutCastStopError();
    
  signals:
    void shoutCastUpdated();
    void shoutCastStoppedError();
};

#endif // _SHOUTCAST_MANAGER_H_
