/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "StreamModel/xspfstreamlist.h"
#include "utilities.h"
#include "debug.h"

#include <QFile>
#include <QXmlStreamReader>

/*
********************************************************************************
*                                                                              *
*    Class XspfStreamList                                                      *
*                                                                              *
********************************************************************************
*/

XspfStreamList::XspfStreamList() {}

XspfStreamList::XspfStreamList(const QString & path) 
{
    m_filename = path;
}

void XspfStreamList::read()
{
    m_mediaitem_list.clear();
    
    Debug::debug() << "-- XspfStreamList read file : " << m_filename;
    QFile file(m_filename);
    
    if(!file.exists()) {
      Debug::debug() << "-- XspfStreamList -> error finding file";
      return;
    }
    
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
      Debug::debug() << "-- XspfStreamList -> error opening";
      return;
    }
    
    QXmlStreamReader xml(&file); 
    MediaItemPtr mi = MediaItemPtr(0);

    
    while(!xml.atEnd() && !xml.hasError()) 
    { 
      xml.readNext();
      if (xml.isStartElement() && xml.name() == "title") 
      {
        m_playlist_name = QString(xml.readElementText());
        //Debug::debug() << "-- XspfStreamList::read -> playlist_name : " << m_playlist_name;
      }
      else if (xml.isStartElement() && xml.name() == "trackList") 
      {
        break;
      }
    }
    
    while(!xml.atEnd() && !xml.hasError()) 
    { 
      xml.readNext();

      if (xml.isStartElement()) 
      {
          if(xml.name() == "track") 
          {
            //! new stream entry
            //Debug::debug() << "-- XspfStreamList -> track start";
            mi = MediaItemPtr(new MediaItem(TYPE_STREAM));
           
          }
          else if (xml.name() == "title" && mi) 
          {
            mi->data[STREAM::name] = QString(xml.readElementText());
          }
          else if (xml.name() == "location" && mi) 
          {
            mi->data[STREAM::url]  = QString(xml.readElementText());
          }
          else if (xml.name() == "category" && mi) 
          {
            mi->data[STREAM::categorie] = QString(xml.readElementText());
          }
      }
      else if (xml.isEndElement() && xml.name() == "track") 
      {
          //Debug::debug() << "-- XspfStreamList -> track end";
          //! if stream OK --> add it
          if (!mi->data.at(STREAM::url).toString().isEmpty() && 
              !mi->data.at(STREAM::name).toString().isEmpty()) 
          {
            m_mediaitem_list.append(mi);
          }
          mi = MediaItemPtr(0);
      }
    } // end while

    if(xml.hasError()) 
    {
      Debug::debug() << "-- XspfStreamList -> error : " << xml.errorString();
    }
    
    qSort(m_mediaitem_list.begin(), m_mediaitem_list.end(), MEDIA::compareStreamCategorie);
}

void XspfStreamList::write()
{
    //! open file to write
    QFile file(m_filename);
    
    if(file.exists()) file.remove();
    if(!file.open(QIODevice::WriteOnly)) return;  
  
    QXmlStreamWriter xml(&file);
    xml.setAutoFormatting(true);
    
    xml.writeStartDocument();
    xml.writeStartElement("playlist");
    xml.writeAttribute("version", "1");
    xml.writeDefaultNamespace("http://xspf.org/ns/0/");
    
    xml.writeTextElement("title", m_playlist_name);
    //Debug::debug() << "-- XspfStreamList::write -> playlist_name : " << m_playlist_name;
    
    xml.writeStartElement("trackList");
    foreach (const MediaItemPtr mi, m_mediaitem_list) 
    {
        if (!mi) continue;
        if (mi->type != TYPE_STREAM) continue;
    
        xml.writeStartElement("track");
        xml.writeTextElement("location", mi->data.at(STREAM::url).toString());
        xml.writeTextElement("title", mi->data.at(STREAM::name).toString());
        xml.writeStartElement("extension");
        xml.writeAttribute("application", "yarock");
        xml.writeTextElement("category", mi->data.at(STREAM::categorie).toString());
        xml.writeEndElement(); //extension
        xml.writeEndElement(); //track
    }
    xml.writeEndElement(); //trackList
    xml.writeEndElement(); //playlist
    xml.writeEndDocument();   
}

void XspfStreamList::setMediaList(QList<MediaItemPtr> list)
{
    m_mediaitem_list.clear();
    m_mediaitem_list = list;
}
