/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _STREAM_SCENE_H_
#define _STREAM_SCENE_H_

#include "CollectionView/graphicitem.h"
#include "CollectionView/graphicitem_menu.h"
#include "EnginePlayer/engineplayer.h"
#include "threadmanager.h"

#include "viewmode.h"


#include <QStringList>
#include <QGraphicsScene>
#include <QtGui>

class VirtualPlayqueue;
class StreamModel;
class IceCastManager;
class ShoutCastManager;


/*
********************************************************************************
*                                                                              *
*    Class StreamScene                                                         *
*                                                                              *
********************************************************************************
*/
class StreamScene : public QGraphicsScene
{
Q_OBJECT
  public: 
    StreamScene(QWidget *parent);
    ~StreamScene();
    void  setVirtualPlayqueue(VirtualPlayqueue  *queue);
    void  setThreadManager(ThreadManager* tm) {m_thread_manager = tm;}
    int   getSceneInfoSize() {return _infoSize;}
    
    QList<CategorieGraphicItem*>  _listCategorie;    
    MAIN::ViewMode                _view_mode;
    
  private :
    ThreadManager            *m_thread_manager;
    StreamModel              *_streamModel;

    IceCastManager           *_iceCastManager;
    ShoutCastManager         *_shoutCastManager;

    VirtualPlayqueue         *_virtualPlaylist;
    EnginePlayer             *_player;
    int                      _infoSize;
    QWidget                  *_parentWidget;
    
    QString                   _currentPlaylist; /* nom du stream playing */
    
    QGraphicsItem            *m_mouseGrabbedItem;
    
    QAction                  *m_remove_duplicate_action;

    GraphicsItemMenu         *m_graphic_item_menu;
    
  protected:
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
    
  private slots :
    void populateStreamScene();
    void populateFavoriteStreamScene();
    void populateWaitingScene(QString text);
    
    void playStream();
    void slot_streams_fetched(QByteArray bytes);
    void enqueueStream(bool is_new_playqueue);
    
    void updateStreamFavorite();

    void slot_item_doubleclick();
    void slot_contextmenu_triggered(ENUM_ACTION_ITEM_MENU);  
    
    void slot_reload_stream_list();
    void slot_remove_duplicate_stream();
    void slot_check_stream();
    void slot_edit_stream_list();

  public slots:
    void populateScene();
    void updateStreamPlaying(const QString& urlName);
    void updateStreamBroken(const QString& urlName = QString());

signals:
    void repopulateStreamScene();
};

#endif // _STREAM_SCENE_H_
