/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "Widget/lineeditclear.h"
#include "Widget/spacer.h"

#include <QtGui>

/*
********************************************************************************
*                                                                              *
*    Class LineEditClear                                                       *
*                                                                              *
********************************************************************************
*/ 
LineEditClear::LineEditClear(QWidget *parent) : QLineEdit(parent)
{
    this->setMinimumWidth(250);
    this->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
  
    m_clearButton = new QToolButton(this);
    m_clearButton->setIcon(QIcon(":/images/blue-cross-32x32.png"));    
    
    m_clearButton->setCursor(Qt::ArrowCursor);
    m_clearButton->setStyleSheet("QToolButton { border: none; padding: 0px; }");
    m_clearButton->hide();
    m_clearButton->setToolTip(tr("Clear"));
    m_clearButton->setFocusPolicy(Qt::NoFocus);
    m_clearButton->setIconSize(QSize(12,12));
    m_clearButton->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );


    QObject::connect(m_clearButton, SIGNAL(clicked()), this, SLOT(clearField()));
    QObject::connect(this, SIGNAL(textChanged(const QString&)), this, SLOT(updateCloseButton(const QString&)));
    QObject::connect(this, SIGNAL(returnPressed()), this, SIGNAL(textfield_entered()));
}

void LineEditClear::setInactiveText(const QString& text)
{
    m_inactiveText = text;
}

void LineEditClear::setEnable(bool e)
{
    this->setEnabled(e);
    m_clearButton->setEnabled(e);
}




void LineEditClear::paintEvent(QPaintEvent *event)
{
    if (this->text().isEmpty() && !hasFocus())
    {
        QLineEdit::paintEvent(event);
        QStyleOptionFrameV2 panel;
        initStyleOption(&panel);
        QRect r = style()->subElementRect(QStyle::SE_LineEditContents, &panel, this);
        QFontMetrics fm = fontMetrics();
        int horizontalMargin = 2;
        QRect lineRect(horizontalMargin + r.x(), r.y() + (r.height() - fm.height() + 1) / 2,
                       r.width() - 2 * horizontalMargin, fm.height());
                       
        QPainter painter(this);
        painter.setPen(palette().brush(QPalette::Disabled, QPalette::Text).color());
        painter.drawText(lineRect, Qt::AlignLeft|Qt::AlignVCenter, m_inactiveText);
    } 
    else 
    {
        QLineEdit::paintEvent(event);
    }
}


void LineEditClear::clearField() 
{
    clear();
    clearFocus();
    emit textfield_entered();
}


void LineEditClear::resizeEvent(QResizeEvent *)
{
    QSize sz = m_clearButton->sizeHint();
    int frameWidth = style()->pixelMetric(QStyle::PM_DefaultFrameWidth);
    m_clearButton->move(rect().right() - frameWidth - sz.width()-3,(rect().bottom() + 1 - sz.height())/2);
}

void LineEditClear::updateCloseButton(const QString& text)
{
    m_clearButton->setVisible(!text.isEmpty());
    emit textfield_changed();
}
